document.addEventListener("DOMContentLoaded", function () {
    function createElement(tag, attributes = {}, text) {
        const element = document.createElement(tag);
        Object.entries(attributes).forEach(([name, value]) => {
            if (value === true) {
                element.setAttribute(name, '');
            } else if (value !== false && value !== null && value !== undefined) {
                element.setAttribute(name, value);
            }
        });
        if (text) element.textContent = text;
        return element;
    }

    // ラジオボタンの初期設定
    function initRadioButtons() {
        const recoveryOptionRadioGroup = document.getElementById("recoveryOptionRadioGroup");
        chrome.storage.sync.get({ recoveryOption: "on" }, (obj) => {
            ["回復ON", "回復OFF"].forEach((label, index) => {
                const value = index === 0 ? "on" : "off";
                const radio = createElement("input", {
                    type: "radio",
                    name: "recoveryOption",
                    value,
                    checked: obj.recoveryOption === value,
                });
                const labelElement = createElement("label", {}, label);

                const row = createElement("tr");
                const radioCell = createElement("td");
                const labelCell = createElement("td");
                radioCell.appendChild(radio);
                labelCell.appendChild(labelElement);
                row.appendChild(radioCell);
                row.appendChild(labelCell);
                recoveryOptionRadioGroup.appendChild(row);
            });
        });
    }

    document.getElementById("saveButton").addEventListener("click", function () {
        // 設定値の取得
        const recoveryOption = document.querySelector(
            'input[name="recoveryOption"]:checked'
        ).value;
        const waitTime = document.getElementById("waitTime").value;
        const buttonSettings = [];

        // ボタンの数を取得する関数
        const getButtonCount = (buttonType) => {
            const buttonCountElement = document.getElementById(`buttonCount${buttonType}`);
            return buttonCountElement ? parseInt(buttonCountElement.value, 10) : 0;
        };

        // 入力されたボタン情報を設定オブジェクトに追加する関数
        const processButtonInputs = (buttonType) => {
            const buttonCount = getButtonCount(buttonType);
            for (let i = 1; i <= buttonCount; i++) {
                const buttonTextElement = document.getElementById(`button${buttonType}${i}Text`);
                const buttonText =
                    buttonTextElement && buttonTextElement.value
                        ? buttonTextElement.value
                        : `リンク${buttonType}${i}`;
                const buttonUrlElement = document.getElementById(`button${buttonType}${i}Url`);
                const buttonUrl = buttonUrlElement && buttonUrlElement.value
                    ? buttonUrlElement.value
                    : "";
                buttonSettings.push({ type: buttonType, text: buttonText, url: buttonUrl });
            }
        };

        // 各ボタンタイプについて設定を保存
        const buttonCountB = getButtonCount("B");
        const buttonCountL = getButtonCount("L");
        const buttonCountR = getButtonCount("R");

        processButtonInputs("B");
        processButtonInputs("L");
        processButtonInputs("R");

        // 設定をストレージに保存
        chrome.storage.sync.set(
            {
                buttonCountB: buttonCountB,
                buttonCountL: buttonCountL,
                buttonCountR: buttonCountR,
                waitTime: waitTime,
                buttonSettings: buttonSettings,
                recoveryOption: recoveryOption,
            },
            function () {
                console.log("Settings saved:", {
                    buttonCountB,
                    buttonCountL,
                    buttonCountR,
                    waitTime,
                    buttonSettings,
                    recoveryOption,
                });
                displayStatusMessage("保存しました", true);
            }
        );
    });

    function displayStatusMessage(message, success) {
        const statusMessageElement = document.getElementById("statusMessage");
        statusMessageElement.textContent = message;
        statusMessageElement.style.color = success ? "green" : "red";
        setTimeout(() => {
            statusMessageElement.textContent = "";
        }, 3000);
    }

    function getDefaultButtonSettings() {
        return Array.from({ length: 10 }, (_, i) => ({
            type: "B",
            text: `リンクB${i + 1}`,
            url: "",
        })).concat(
            Array.from({ length: 10 }, (_, i) => ({
                type: "L",
                text: `リンクL${i + 1}`,
                url: "",
            }))
        ).concat(
            Array.from({ length: 10 }, (_, i) => ({
                type: "R",
                text: `リンクR${i + 1}`,
                url: "",
            }))
        );
    }

    // UI初期化関数
    function initUI(currentSettings) {
        document.getElementById("buttonCountB").value = currentSettings.buttonCountB;
        document.getElementById("buttonCountL").value = currentSettings.buttonCountL;
        document.getElementById("buttonCountR").value = currentSettings.buttonCountR;
        document.getElementById("waitTime").value = currentSettings.waitTime;

        ["B", "L", "R"].forEach((buttonType) => {
            generateButtonInputs(
                buttonType,
                currentSettings[`buttonCount${buttonType}`],
                currentSettings.buttonSettings
            );
        });
    }

    // ボタン入力エリアの生成
    function generateButtonInputs(buttonType, buttonCount, buttonSettings) {

        console.log(`buttonType: ${buttonType}, buttonSettings:`, buttonSettings); // 新しいデバッグ文

        const buttonTextInputs = document.getElementById(`buttonTextInputs${buttonType}`);
        if (!buttonTextInputs) {
            console.error(`buttonTextInputs${buttonType} not found.`);
            return;
        }

        const filteredSettings = buttonSettings.filter((setting) => setting.type === buttonType);

        buttonTextInputs.innerHTML = "";
        for (let i = 1; i <= buttonCount; i++) {
            const setting = filteredSettings[i - 1];
            console.log(`setting:`, setting); // 新しいデバッグ文

            const initialText = setting ? setting.text : `リンク${buttonType}${i}`;
            const initialUrl = setting ? setting.url : "";

            const inputText = createElement("input", {
                id: `button${buttonType}${i}Text`,
                type: "text",
            });
            inputText.value = initialText;

            const inputUrl = createElement("textarea", {
                id: `button${buttonType}${i}Url`,
                rows: "3",
                cols: "20",
            });
            inputUrl.textContent = initialUrl;

            const labelCell = createElement("td", {}, `ボタン${buttonType}${i}`);
            const row = createElement("tr");
            const textCell = createElement("td");
            const urlCell = createElement("td");
            textCell.appendChild(inputText);
            urlCell.appendChild(inputUrl);
            row.appendChild(labelCell);
            row.appendChild(textCell);
            row.appendChild(urlCell);
            buttonTextInputs.appendChild(row);
        }
    }

    // ラジオボタンの初期化
    initRadioButtons();

    // ストレージから現在の設定を読み込んでUIを初期化する
    chrome.storage.sync.get(
        {
            buttonCountB: 5,
            buttonCountL: 10,
            buttonCountR: 10,
            waitTime: 300,
            buttonSettings: getDefaultButtonSettings(),
            recoveryOption: "on",
        },
        initUI
    );

    // 設定アップロード、ダウンロード
    document.getElementById("loadSettingsButton").addEventListener("click", function () {
        const configId = document.getElementById("configIdInput").value;
        const url = `http://192.168.2.150:3007/rise/download/${configId}`;

        // 設定の読み込み
        loadSettingsFromCsvUrl(url);
    });


    document.getElementById("uploadSettingsButton").addEventListener("click", function () {
        const settings = [
            // ここに入力された値を含む設定オブジェクトを追加
        ];
        const url = "http://192.168.2.150:3007/rise/upload";

        // 設定のアップロード
        saveSettingsToCsvUrl(url, settings);
    });

    // 入力された設定データをURLにCSV形式で保存する関数
    function saveSettingsToCsvUrl(url, settings) {
        // 設定データをCSV形式に変換
        const csvText =
            "config_id,setting_name,setting_type,setting_key,setting_value\n" +
            settings
                .map(
                    (setting) =>
                        `${setting.config_id},${setting.setting_name},${setting.setting_type},${setting.setting_key},${setting.setting_value}`
                )
                .join("\n");

        // CSVデータをPOSTリクエストで送信
        return fetch(url, {
            method: "POST",
            headers: {
                "Content-Type": "text/csv",
            },
            body: csvText,
        })
            .then((response) => {
                if (!response.ok) {
                    // 通信エラーが発生した時の処理
                    throw new Error("Network response was not ok");
                }
                // 送信完了
                console.log("Settings saved in CSV format to server");
            })
            .catch((error) => {
                // 通信に関するエラー処理
                console.error("There was a problem with the fetch operation:", error);
            });
    }


    // CSVをパースしてネストされたオブジェクトのリストに変換する関数
    function parseCsv(text, delimiter = ",") {
        const rows = text.split("\n").filter((row) => row.trim().length > 0);
        const header = rows.shift().split(delimiter).map(s => s.trim());
        const data = rows.map((row) =>
            row.split(delimiter).reduce((acc, cell, i) => {
                acc[header[i]] = cell.trim().replace(/\r$/, "");
                return acc;
            }, {})
        );
        return data;
    }

    // URLからCSV形式の設定を読み込む関数
    function loadSettingsFromCsvUrl(url) {
        return fetch(url)
            .then((response) => {
                if (!response.ok) {
                    throw new Error("Network response was not ok");
                }
                return response.text();
            })
            .then((csvText) => {
                const settings = parseCsv(csvText);
                console.log("parseCSV", settings);

                const properties = settings
                    .filter((setting) => setting.setting_type === "property")
                    .reduce((acc, property) => {
                        acc[property.setting_key] = property.setting_value;
                        return acc;
                    }, {});
                const buttonSettings = settings.filter(
                    (setting) => setting.setting_type === "button"
                );

                const buttonSettingsMap = buttonSettings.reduce((acc, setting) => {
                    const type = setting.setting_key.charAt(6);
                    const indexStr = setting.setting_key.match(/(\d+)/);
                    const index = indexStr ? parseInt(indexStr[0], 10) - 1 : -1;
                    const key = `${type}${index}`;
                
                    if (index < 0) {
                        return acc;
                    }
                
                    if (!acc[type]) {
                        acc[type] = {};
                    }
                
                    if (!acc[type][index]) {
                        acc[type][index] = {
                            type: type,
                            text: "",
                            url: "",
                        };
                    }
                
                    if (setting.setting_key.endsWith("Text")) {
                        acc[type][index].text = setting.setting_value;
                    } else if (setting.setting_key.endsWith("Url")) {
                        acc[type][index].url = setting.setting_value;
                    }
                
                    return acc;
                }, {});

                const buttonSettingsList = [
                    ...(Object.values(buttonSettingsMap["B"] || {})),
                    ...(Object.values(buttonSettingsMap["L"] || {})),
                    ...(Object.values(buttonSettingsMap["R"] || {}))
                ];

                console.log("properties", properties);
                console.log("buttonSettingsList: ", buttonSettingsList);

                const currentSettings = {
                    buttonCountB: parseInt(properties.buttonCountB, 10),
                    buttonCountL: parseInt(properties.buttonCountL, 10),
                    buttonCountR: parseInt(properties.buttonCountR, 10),
                    waitTime: parseInt(properties.waitTime, 10),
                    recoveryOption: properties.recoveryOption,
                    buttonSettings: buttonSettingsList,
                };

                initUI(currentSettings);
            })
            .catch((error) => {
                console.error("There was a problem with the fetch operation:", error);
            });
    }
});
